# ECS MCP Server LLM Testing Framework

This directory contains a framework for testing the ECS MCP Server's troubleshooting capabilities using LLM (Large Language Model) tools. The framework creates reproducible failure scenarios in AWS ECS that can be used to evaluate how well Cline diagnoses and resolves common ECS deployment issues.

## Framework Structure

```
tests/llm_testing/
├── README.md                        # This file
├── run_tests.sh                     # Main script to run test scenarios
├── scenarios/                       # Test scenarios
│   ├── 01_cloudformation_failure/    # CloudFormation stack failure scenario
│   │   ├── 01_create.sh              # Creates a failing CloudFormation stack
│   │   ├── 02_validate.sh            # Validates the stack has failed as expected
│   │   ├── 03_prompts.txt            # Different prompts to test with Cline
│   │   ├── 04_evaluation.md          # Evaluation criteria for Cline's responses
│   │   └── 05_cleanup.sh             # Cleans up created resources
│   │
│   ├── 02_service_failure/          # Service-level image pull failure scenario
│   │   ├── 01_create.sh              # Creates a failing ECS service
│   │   ├── 02_validate.sh            # Validates the service has failed tasks
│   │   ├── 03_prompts.txt            # Different prompts to test with Cline
│   │   ├── 04_evaluation.md          # Evaluation criteria for Cline's responses
│   │   └── 05_cleanup.sh             # Cleans up created resources
│   │
│   └── 03_task_exit_failure/        # Task exit code failure scenario
│       ├── 01_create.sh              # Creates an ECS task that exits with an error
│       ├── 02_validate.sh            # Validates the task has exited with error code
│       ├── 03_prompts.txt            # Different prompts to test with Cline
│       ├── 04_evaluation.md          # Evaluation criteria for Cline's responses
│       └── 05_cleanup.sh             # Cleans up created resources
│
└── utils/                           # Shared utilities
    ├── aws_helpers.sh                # Common AWS CLI helper functions
    └── evaluation_template.md        # Template for evaluating responses
```

## Prerequisites

1. AWS CLI installed and configured with appropriate permissions
2. jq command-line tool for JSON processing
3. Proper AWS IAM permissions to create and manage ECS resources

## How to Use

### Running Tests

1. Execute the main script:
   ```bash
   cd src/ecs-mcp-server/tests/llm_testing
   ./run_tests.sh
   ```

2. Select a scenario to test or run all scenarios sequentially

3. The script will:
   - Create resources with intentional failures
   - Validate the failures match expectations
   - Display prompts you can use to test Cline
   - Offer to clean up resources when done

### Individual Scenario Execution

You can also run each scenario individually:

```bash
cd src/ecs-mcp-server/tests/llm_testing/scenarios/01_cloudformation_failure
./01_create.sh
# Wait for resources to be created and fail
./02_validate.sh
# Test with Cline using prompts from 03_prompts.txt
# When testing is complete
./05_cleanup.sh
```

## Available Test Scenarios

### 1. CloudFormation Stack Failure

This scenario creates a CloudFormation stack that fails due to missing NetworkConfiguration in an ECS service. It tests Cline's ability to:
- Diagnose CloudFormation template issues
- Find specific error messages in stack events
- Understand and explain ECS Fargate networking requirements
- Provide appropriate solutions

### 2. ECS Service Image Pull Failure

This scenario creates an ECS service that fails to start tasks due to a non-existent container image. It tests Cline's ability to:
- Diagnose service-level failures in ECS
- Understand Docker image pull errors
- Find and interpret service events
- Suggest correct solutions for image issues

### 3. ECS Task Exit Code Failure

This scenario runs an ECS task that exits with error code 1 due to a missing environment variable. It tests Cline's ability to:
- Find and interpret task-level failures
- Locate and analyze container logs
- Understand application-level issues
- Recommend appropriate configuration changes

## Testing with Cline

After setting up a scenario, use the prompts provided in the `03_prompts.txt` file for that scenario. These prompts vary in complexity and user knowledge level to test Cline's ability to adapt explanations appropriately.

Replace the placeholders in the prompts with the actual resource names generated by the scripts.

## Evaluation

Each scenario includes an evaluation template in `04_evaluation.md` that can be used to assess Cline's performance. The evaluation criteria include:
- Problem identification
- Root cause analysis
- Solution quality
- Educational value

## Resource Management

All scripts generate random IDs for resource names to avoid conflicts with previous test runs. The cleanup scripts will automatically find and remove all resources created by the test scenarios.

## Important Notes

- The tests create actual AWS resources that may incur costs
- Always run the cleanup script after testing to remove any created resources
- AWS credentials must be properly configured with appropriate permissions

## Adding New Scenarios

To add a new scenario:

1. Create a new directory under `scenarios/` with the appropriate naming convention
2. Create the standard files: `01_create.sh`, `02_validate.sh`, `03_prompts.txt`, `04_evaluation.md`, and `05_cleanup.sh`
3. Use the shared utilities in the `utils/` directory
4. Test thoroughly to ensure the scenario creates reproducible failures

## Troubleshooting

If resources fail to clean up properly, you can use the cleanup scripts with no arguments to find and remove all test resources matching the naming patterns used by the tests.
